// Copyright (c) 2011 Bryan Schumaker.
#include <notify.h>
#include <audio.h>
#include <print.h>
#include <fs.h>
#include "audio.h"

static GstState cur_state = GST_STATE_READY;

static void notify_state_change(GstState state)
{
	if (state == GST_STATE_PLAYING)
		libsaria::notify(PLAYING, NULL);
	else if (state == GST_STATE_PAUSED)
		libsaria::notify(PAUSED, NULL);
}

static bool change_state(GstState new_state)
{
	GstStateChangeReturn ret;
	ret = gst_element_set_state(GST_ELEMENT(player), new_state);

	switch(ret) {
		case GST_STATE_CHANGE_SUCCESS:
		case GST_STATE_CHANGE_ASYNC:
			cur_state = new_state;
			notify_state_change(cur_state);
			return true;
		default:
			return false;
	}
}

void load_file(GstElement *playbin, string file, GstState state)
{
	if (file == "" || !libsaria::exists(file))
		return;

	string uri = "file://" + file;
	println("Loading uri: " + uri);
	/* Set pipeline to the requested state */
	change_state(GST_STATE_READY);
	g_object_set(G_OBJECT(playbin), "uri", uri.c_str(), NULL);
	change_state(state);
}

namespace libsaria
{

	void audio::play()
	{
		change_state(GST_STATE_PLAYING);
	}

	void audio::pause()
	{
		change_state(GST_STATE_PAUSED);
	}

	void audio::toggle_play()
	{
		if (cur_state == GST_STATE_PLAYING)
			pause();
		else
			play();
	}

	GstState audio::get_state()
	{
		return cur_state;
	}

	void audio::stop()
	{
		pause();
		seek_to(0);
	}

	void audio::load(string file, bool play)
	{
		GstState state = GST_STATE_PLAYING;
		if (!play)
			state = GST_STATE_PAUSED;
		load_file(player, file, state);
	}

};
