// Copyright (c) 2012 Bryan Schumaker.
#include <library.h>
#include <notify.h>
#include <track.h>
#include <print.h>
#include <deck.h>
#include <list.h>
#include <idle.h>
#include <fs.h>
#include "library.h"

static unsigned int MAX_SCAN = 20;

/*
 * Scan the library path and remove files that don't exist anymore
 */
class ValidateTask : public IdleTask
{
	private:
		struct libsaria::library::Path *path;

	public:
		ValidateTask(struct libsaria::library::Path *);
		~ValidateTask();
		void run_task();
};

ValidateTask::ValidateTask(struct libsaria::library::Path *p)
{
	path = p;
}

ValidateTask::~ValidateTask()
{
}

void ValidateTask::run_task()
{
	libsaria::Track *track;
	libsaria::ListItem<libsaria::Track> *it;

	for (it = path->tracks.first(); it != path->tracks.end(); it = it->next()) {
		track = &it->get_value();
		if (!libsaria::exists((*track)[TRACK_FILEPATH])) {
			println("Reaping: " + (*track)[TRACK_FILEPATH]);
			lib_playlist.remove_track(&it->get_value());
			it = path->tracks.erase(it);
			path->data_state = DIRTY;
		}
	};

	libsaria::notify(PATH_UPDATED, path);
	save_path(path);
}


/*
 * Tag each file in file_list and add to the library path
 */
class ScanTask : public IdleTask
{
	private:
		struct libsaria::library::Path *path;
		list<string> file_list;

	public:
		ScanTask(struct libsaria::library::Path *, list<string> &);
		~ScanTask();
		void run_task();
};

ScanTask::ScanTask(struct libsaria::library::Path *p, list<string> &files)
{
	list<string>::iterator it;
	path = p;

	if (files.size() <= MAX_SCAN)
		it = files.end();
	else {
		it = files.begin();
		for (unsigned int i = 0; i < MAX_SCAN; i++)
			it++;
	}

	file_list.splice(file_list.begin(), files, files.begin(), it);
}

ScanTask::~ScanTask()
{
}

static bool check_file(libsaria::Track &track, void *data)
{
	return track[TRACK_FILEPATH] == *(string *)data;
}

void ScanTask::run_task()
{
	list<string>::iterator it;
	libsaria::ListItem<libsaria::Track> *li;

	for (it = file_list.begin(); it != file_list.end(); it++) {
		/*
		 * TODO: What I really should do for this check is build up
		 * a path component tree right before scanning, and then walk
		 * the tree to check if the path exists.  It'll be much faster
		 * than having to walk the same list over and over...
		 */
		if (path->tracks.find_item(check_file, &(*it)))
			continue;

		try {
			li = path->tracks.push_back(libsaria::Track(*it, path));
			lib_playlist.push_back(&li->get_value());
			path->data_state = DIRTY;
		} catch (string message) {
			println(message);
		}
	}

	libsaria::notify(PATH_UPDATED, path);
}


/*
 * Read a directory and create ScanTasks for chunks of files
 */
class ReaddirTask : public IdleTask
{
	private:
		struct libsaria::library::Path *path;

	public:
		ReaddirTask(struct libsaria::library::Path *path);
		~ReaddirTask();
		void run_task();
};

ReaddirTask::ReaddirTask(struct libsaria::library::Path *p)
{
	path = p;
}

ReaddirTask::~ReaddirTask()
{
}

void ReaddirTask::run_task()
{
	list<string> file_list;
	list<string>::iterator it;
	unsigned int i;
	ScanTask *task;

	libsaria::list_dir(path->path, file_list);
	println("Found: %d files to scan", file_list.size());

	i = 0;
	while (file_list.size() > 0) {
		task = new ScanTask(path, file_list);
		task->queue();
		if ((i++ % 50) == 0)
			save_path(path);
	}
	save_path(path);
}

void do_update_path(struct libsaria::library::Path *path)
{
	println("Updating path: " + path->path);

	if (!libsaria::exists(path->path)) {
		libsaria::library::delete_path(path);
		return;
	}

	ValidateTask *validate = new ValidateTask(path);
	ReaddirTask *task = new ReaddirTask(path);
	validate->queue();
	task->queue();
}
