// Copyright (c) 2012 Bryan Schumaker.
#ifndef LIBSARIA_LIST_CPP
#define LIBSARIA_LIST_CPP

#include <list.h>
#include <print.h>

#include <assert.h>

namespace libsaria
{

	template <class T>
	List<T>::List()
	{
		count = 0;
		head.list = this;
	}

	/*
	 * This just claims to be a copy constructor.
	 * A real one should be implemented later if it's needed
	 */
	template <class T>
	List<T>::List(const List<T> &orig)
	{
		count = 0;
		head.list = this;
	}

	template <class T>
	List<T>::~List()
	{
		while (count > 0)
			erase(head.next_item);
	}

	template <class T>
	ListItem<T> *List<T>::push_front(T item)
	{
		ListItem<T> *node;
		node = new ListItem<T>(item, this, &head, first());
		count++;
		return node;
	}

	template <class T>
	ListItem<T> *List<T>::push_back(T item)
	{
		ListItem<T> *node;
		node = new ListItem<T>(item, this, head.prev_item, &head);
		count++;
		return node;
	}

	template <class T>
	T List<T>::pop_front()
	{
		T item;
		if (count == 0)
			item = head.value;
		else {
			item = first()->value;
			erase(first());
		}
		return item;
	}

	template <class T>
	ListItem<T> *List<T>::erase(ListItem<T> *node)
	{
		ListItem<T> *prev = node->prev_item;
		ListItem<T> *next = node->next_item;

		prev->next_item = next;
		next->prev_item = prev;
		count--;
		delete node;

		return prev;
	}

	template <class T>
	unsigned int List<T>::size()
	{
		return count;
	}

	template <class T>
	ListItem<T> *List<T>::first()
	{
		return head.next_item;
	}

	template <class T>
	ListItem<T> *List<T>::end()
	{
		return &head;
	}

	template <class T>
	void List<T>::for_each_item(void (*func)(T &, void *), void *data)
	{
		for (ListItem<T> *it = first(); it != end(); it = it->next())
			func(it->value, data);
	}

	template <class T>
	ListItem<T> *List<T>::find_item(bool (*func)(T &, void *), void *data)
	{
		ListItem<T> *it;
		for (it = first(); it != end(); it = it->next()) {
			if (func(it->value, data))
				return it;
		}
		return NULL;
	}

}

#endif /* LIBSARIA_LIST_CPP */
