// Copyright (c) 2012 Bryan Schumaker.
#include <playlist.h>
#include <library.h>
#include <track.h>
#include <print.h>
#include <deck.h>
#include <idle.h>

#include <stdlib.h>
#include <fstream>
#include <sstream>
using namespace std;

#define PLAYLIST_CURRENT_VERSION 3 // 9 / 14 / 2012

static string plistdir = "playlist";

static string number_to_filename(unsigned int id)
{
	stringstream s;
	s << id;
	return s.str();
}

static void save_playlist(ofstream &stream, void *plist)
{
	libsaria::Playlist *playlist = (libsaria::Playlist *)plist;
	playlist->save(stream);
}

void read_plist(ifstream &stream)
{
	unsigned int version, type, size;
	unsigned int libid, trackid, flags = 0;
	bool disabled = false;
	bool random = false;
	bool sorted = false;
	list<libsaria::Track *> tracks;
	libsaria::Track *track;

	stream >> version;
	if (version < 3)
		stream >> type;
	stream >> size >> disabled;

	if (version >= 3) {
		stream >> random;
		stream >> sorted;
	} else if (type == 0) { /* Old PLIST_SET type */
		random = true;
		sorted = true;
	}

	println("playlist size: %u disabled: %d random: %d sorted: %d",
			size, disabled, random, sorted);

	for (unsigned int i = 0; i < size; i++) {
		stream >> libid >> trackid;
		track = libsaria::library::lookup(libid, trackid);
		if (track)
			tracks.push_back(track);
	}

	if (tracks.size() > 0) {
		if (disabled)
			flags |= PL_DISABLED;
		if (random)
			flags |= PL_RANDOM;
		if (sorted)
			flags |= PL_SORTED;
		libsaria::deck::new_playlist(tracks, flags, false);
	}
}

namespace libsaria
{

	void Playlist::save(ofstream &stream)
	{
		vector<Track *>::iterator it;

		stream << PLAYLIST_CURRENT_VERSION << "\n";
		stream << plist.size()             << " ";
		stream << check_flag(PL_DISABLED)  <<" ";
		stream << check_flag(PL_RANDOM)    << " ";
		stream << check_flag(PL_SORTED)    << "\n";

		for (it = plist.begin(); it != plist.end(); it++) {
			stream << (**it)[TRACK_LIBID]	<< " ";
			stream << (**it)[TRACK_ID]	<< "\n";
		}
	}

	void Playlist::schedule_save()
	{
		string filepath;

		if (number < 0)
			return;
		else if (get_size() == 0) {
			rm_file();
			return;
		}

		filepath = plistdir + "/" + number_to_filename(number);
		app::save(filepath, save_playlist, &data_state, this);
	}

	void Playlist::rm_file()
	{
		string filepath;

		if (number < 0)
			return;

		idle::cancel_all(this);
		filepath = plistdir + "/" + number_to_filename(number);
		app::rm(filepath);
	}

	void deck::load_all()
	{
		app::read_numdir(plistdir, read_plist);
	}

}; /* Namespace: libsaria */
