// Copyright (c) 2011 Bryan Schumaker.
#include <library.h>
#include <track.h>
#include <print.h>

/*
 * Return positive value if s1 > s2
 * Return negative value if s1 < s2
 * Return 0 s1 == s2
 */
static inline int compare_strings(string *s1, string *s2)
{
	/* List empty strings last */
	if (*s1 == "")
		return 1;
	if (*s2 == "")
		return -1;
	return s1->compare(*s2);
}

namespace libsaria
{

	/* Return true if this < cmp_track */
	bool Track::operator<(Track &cmp_track)
	{
		int cmp;

		/* Compare artists */
		cmp = compare_strings(artist_lc, cmp_track.artist_lc);
		if (cmp < 0)
			return true;
		else if (cmp > 0)
			return false;

		/* Compare albums */
		cmp = compare_strings(album_lc, cmp_track.album_lc);
		if (cmp < 0)
			return true;
		else if (cmp > 0)
			return false;

		/* Compare tracks */
		return track < cmp_track.track;
	}

	const string &Track::operator[](track_sprop_t property)
	{
		switch (property) {
		case TRACK_FILEPATH:
			return filepath;
		case TRACK_TITLE:
			return title;
		case TRACK_ARTIST:
			return artist;
		case TRACK_ALBUM:
			return album;
		case TRACK_COMMENT:
			return comment;
		case TRACK_GENRE:
			return genre;
		default: /* TRACK_LENSTR */
			return lenstr;
		}
	}

	const unsigned int &Track::operator[](track_uiprop_t property)
	{
		switch (property) {
		case TRACK_ID:
			return id;
		case TRACK_LIBID:
			return path->id;
		case TRACK_YEAR:
			return year;
		case TRACK_TRACK:
			return track;
		case TRACK_COUNT:
			return count;
		case TRACK_LAST_DAY:
			return last_day;
		case TRACK_LAST_MONTH:
			return last_month;
		default: /* TRACK_LAST_YEAR */
			return last_year;
		}
	}

	const int &Track::operator[](track_iprop_t property)
	{
		switch (property) {
		case TRACK_LENGTH:
			return length;
		case TRACK_BITRATE:
			return bitrate;
		case TRACK_SAMPLE:
			return sample;
		default: /* TRACK_CHANNELS */
			return channels;
		}
	}

	const bool &Track::operator[](track_bprop_t property)
	{
		switch (property) {
		default: /* TRACK_BANNED */
			return banned;
		}
	}

}
